/*
 * casocklib - An asynchronous communication library for C++
 * ---------------------------------------------------------
 * Copyright (C) 2010 Leandro Costa
 *
 * This file is part of casocklib.
 *
 * casocklib is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation, either version 3 of
 * the License, or (at your option) any later version.
 *
 * casocklib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with casocklib. If not, see <http://www.gnu.org/licenses/>.
 */

/*!
 * \file casock/rpc/protobuf/client/RPCCall.h
 * \brief [brief description]
 * \author Leandro Costa
 * \date 2010
 *
 * $LastChangedDate$
 * $LastChangedBy$
 * $Revision$
 */

#ifndef __CASOCKLIB__CASOCK_RPC_PROTOBUF_CLIENT__RPC_CALL_H_
#define __CASOCKLIB__CASOCK_RPC_PROTOBUF_CLIENT__RPC_CALL_H_

#include <time.h>
#include "casock/util/Lockable.h"
#include "casock/util/Logger.h"


namespace google {
  namespace protobuf {
    class Message;
    class Closure;
    class RpcController;
  }
}

namespace casock {
  namespace rpc {
    namespace protobuf {
      namespace api {
        class RpcResponse;
      }
    }

    namespace protobuf {
      namespace client {
        class RPCReaderHandler;
        using casock::rpc::protobuf::api::RpcResponse;

        class RPCCall : public casock::util::Lockable
        {
          public:
            RPCCall (::google::protobuf::Message* pResponse, ::google::protobuf::RpcController* pController, ::google::protobuf::Closure* pClosure,int timeOut=9 )
              : mpResponse (pResponse), mpController (pController), mpClosure (pClosure), mpRpcResponse (NULL)
            ,mTimeoutMilliSeconds((long long)1000*(long long)timeOut)
            {
            	LOGMSG (LOW_LEVEL, "%s - ......this:[%p]..timeout=[%lld]..\n", __FUNCTION__,this,mTimeoutMilliSeconds);
            }
            virtual ~RPCCall(){
            	//LOGMSG (LOW_LEVEL, "%s - calling.................\n", __PRETTY_FUNCTION__);
            }

          public:

            ::google::protobuf::Message* response () { return mpResponse; }
            ::google::protobuf::RpcController* controller () { return mpController; }
            ::google::protobuf::Closure* closure () { return mpClosure; }

            void setRpcResponse (RpcResponse* pRpcResponse) { mpRpcResponse = pRpcResponse; }
            RpcResponse* rpcResponse () { return mpRpcResponse; }
            void setTimeoutSeconds(int iSecond){
            	LOGMSG (LOW_LEVEL, "%s - seconds=[%ld][%ld]\n", __PRETTY_FUNCTION__,iSecond);
            	setTimeoutMilliSeconds((long long)1000 * iSecond);
            }
            void setTimeoutMilliSeconds(long long iMilliSecond){
            	LOGMSG (LOW_LEVEL, "%s - iMilliSecond=[%ld]\n", __PRETTY_FUNCTION__,iMilliSecond);
            	this->mTimeoutMilliSeconds = iMilliSecond;
            }
            bool isTimeOut(){
            	struct timeval now ;
            	usleep(1000);
            	gettimeofday(&now, NULL);
            	long long ims = (now.tv_sec - mTimer.tv_sec)* (long long)1000 + (now.tv_usec - mTimer.tv_usec)/(long long)1000; //milli seconds
            	LOGMSG (LOW_LEVEL, "%s - [%p]passed [%lld]ms compare timeout=[%lld]\n", __PRETTY_FUNCTION__,this,ims,mTimeoutMilliSeconds);
            	//ims < 0 means system time changed, should be not timeout
            	bool bRtn=(ims > 0 && ims >= (this->mTimeoutMilliSeconds));
            	LOGMSG (LOW_LEVEL, "%s - returns [%d]\n", __PRETTY_FUNCTION__,bRtn);
            	return bRtn;
            }
            void begin(){
            	//this->mTimer = time(NULL);
            	gettimeofday(&mTimer,NULL);
            }

          private:
            ::google::protobuf::Message* mpResponse;
            ::google::protobuf::RpcController* mpController;
            ::google::protobuf::Closure* mpClosure;
            long long mTimeoutMilliSeconds;
            struct timeval mTimer;
            RpcResponse* mpRpcResponse;

        };
      }
    }
  }
}

#endif // __CASOCKLIB__CASOCK_RPC_PROTOBUF_CLIENT__RPC_CALL_H_
